@@ -7,7 +7,18 @@ if [[ -z ${BINARY} ]] || [[ -z ${VERSION} ]]; then
 	exit 1
 fi
 
-ARCHS=(darwin-amd64 linux-amd64 linux-arm64 linux-ppc64le linux-armv6 freebsd-amd64 windows-amd64)
+ARCHS=(
+  darwin-amd64
+  darwin-arm64
+  linux-amd64
+  linux-arm64
+  linux-armv5
+  linux-armv6
+  linux-armv7
+  linux-ppc64le
+  freebsd-amd64
+  windows-amd64
+)
 
 mkdir -p release
 
@@ -19,8 +30,9 @@ for ARCH in "${ARCHS[@]}"; do
 	GO_ARCH=$(echo $ARCH | awk -F- '{print $2}')
 
 	# Create architecture specific binaries
-	if [[ ${GO_ARCH} == "armv6" ]]; then
-		GO111MODULE=on GOOS=${GO_OS} GOARCH=arm GOARM=6 CGO_ENABLED=0 go build -ldflags="-X main.VERSION=${VERSION}" \
+	if [[ ${GO_ARCH} == armv* ]]; then
+	  GO_ARM=$(echo $GO_ARCH | awk -Fv '{print $2}')
+		GO111MODULE=on GOOS=${GO_OS} GOARCH=arm GOARM=${GO_ARM} CGO_ENABLED=0 go build -ldflags="-X main.VERSION=${VERSION}" \
 			-o release/${BINARY}-${VERSION}.${ARCH}/${BINARY} .
 	else
 		GO111MODULE=on GOOS=${GO_OS} GOARCH=${GO_ARCH} CGO_ENABLED=0 go build -ldflags="-X main.VERSION=${VERSION}" \
